unit FormMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants, FMX.Types, FMX.Controls, FMX.Forms,
  FMX.Graphics, FMX.Dialogs, FMXTee.Engine, FMXTee.Series, FMX.Controls.Presentation, FMX.StdCtrls, FMXTee.Procs,
  FMXTee.Chart, uGuiThreads, FMX.Edit, FMX.EditBox, FMX.SpinBox, System.DateUtils, System.Diagnostics;

type
  TfrmMain = class(TForm)
    chMain: TChart;
    pnlInfo: TPanel;
    pnlBottom: TPanel;
    SeriesLiveTracking: TFastLineSeries;
    pbStartStop: TButton;
    spValuesToKeep: TSpinBox;
    lblValuesToKeep: TLabel;
    lblPerformanceInfo: TLabel;
    lblNumOfElementsInSeries: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pbStartStopClick(Sender: TObject);
    procedure spValuesToKeepChange(Sender: TObject);
  private
    { Private declarations }
    NumOfSecondsToViewInChart: integer;
    procedure DeleteSeriesElements;
    procedure ScrollChart;

  public
    { Public declarations }
    procedure AddAndScrollChart(aPointInTime: TDateTime; aSignalValue: double);
  end;

var
  frmMain: TfrmMain;
  aStopWatch: TStopwatch;
  LastExecution: Int64;

implementation

{$R *.fmx}

{ TfrmMain }
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  {$IFDEF ANDROID}
  FullScreen := true;
  {$ENDIF}

  //
  LastExecution := 0;
  NumOfSecondsToViewInChart := 30;

  //Chart
  chMain.AutoRepaint := false;
  chMain.Legend.Visible := False;
  chMain.Frame.Visible := False;
  chMain.View3D := False;
  chMain.Title.Visible := False;

  chMain.BottomAxis.Automatic := False;
  chMain.BottomAxis.AutomaticMaximum := False;
  chMain.BottomAxis.AutomaticMinimum := False;
  chMain.BottomAxis.DateTimeFormat := 'hh:MM:ss';
  chMain.BottomAxis.RoundFirstLabel := false;

  chMain.LeftAxis.Automatic := False;
  chMain.LeftAxis.AutomaticMaximum := False;
  chMain.LeftAxis.AutomaticMinimum := False;
  chMain.LeftAxis.Maximum := 102;
  chMain.LeftAxis.Increment := 20;
  chMain.LeftAxis.AxisValuesFormat := '0%';


  //TrackingSeries
  SeriesLiveTracking.XValues.DateTime := true;
  SeriesLiveTracking.LinePen.Width := 1.4;
  SeriesLiveTracking.LinePen.Fill.Color := StrToInt('0xFF4466A3');
  SeriesLiveTracking.TreatNulls := tnIgnore;
  SeriesLiveTracking.XValues.Order := loNone;

end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  //
end;

procedure TfrmMain.AddAndScrollChart(aPointInTime: TDateTime; aSignalValue: double);
begin

  //Do chart stuff
  SeriesLiveTracking.AddXY(aPointInTime, aSignalValue);
  DeleteSeriesElements;
  ScrollChart;
  chMain.Repaint;

  //Timing
  lblPerformanceInfo.Text := Format('Main Thread is called every: %d MilliSec', [aStopWatch.ElapsedMilliseconds - LastExecution]);
  LastExecution := aStopWatch.ElapsedMilliseconds;

end;


procedure TfrmMain.pbStartStopClick(Sender: TObject);
begin
  if IsAddingAndScrolling then begin
    StopAddAndScroll;
  end else begin
    aStopWatch := TStopwatch.StartNew;
    StartAddAndScroll;
  end;
end;

procedure TfrmMain.DeleteSeriesElements;
begin
  while SeriesLiveTracking.XValues.Count >  trunc(spValuesToKeep.Value) do begin
    SeriesLiveTracking.Delete(0);
  end;

  //Display num of elements
  lblNumOfElementsInSeries.Text := Format('Number of elements in Series: %d', [SeriesLiveTracking.Count]);
end;

procedure TfrmMain.ScrollChart;
begin
  chMain.BottomAxis.SetMinMax(IncSecond(now, -NumOfSecondsToViewInChart), now);
end;

procedure TfrmMain.spValuesToKeepChange(Sender: TObject);
begin
  //Just add some random points to series so that we do not have to wait for it .....
  //Just for demo purpose
  if SeriesLiveTracking.Count < trunc(spValuesToKeep.Value) then begin
    SeriesLiveTracking.FillSampleValues(trunc(spValuesToKeep.Value));
  end;

end;

end.
